;********************************************************************
;* Schneider Joyce                                                  *
;*                                                                  *
;* PltPix - Draws a single pixel                                    *
;*          X0            X coordinate                              *
;*          Y0 low byte   Y coordinate                              *
;*          Y0 high byte  mode (0=set, 1=reset, 2=invert)           *
;* Line   - Draws a line                                            *
;*          X0            X coordinate starting point               *
;*          Y0            Y-Koordinate starting point               *
;*          X1            X-Koordinate end point                    *
;*          Y1            Y-Koordinate end point                    *
;* Symbol - Symbol                                                  *
;* Get    - Get                                                     *
;* ScrAcc - Screen access                                           *
;********************************************************************
;
;       .Z80
;       CSEG
;
	org	0F200h

xbios	equ	0FC5Ah		;Einsprungadresse fuer XBIOS
screen	equ	00E9h		;Nummer der XBIOS-Funktion

PltPix: ld	bc,start1
	call	xbios
	dw	screen		;XBIOS-Funktion
	ret
start1: ld	de,(X0)
	ld	hl,(Y0)
	ld	a,03h
	and	d
	ld	d,a
	push	hl
	ld	h,00h
	add	hl,hl
	ld	bc,0b600h
	add	hl,bc
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	ld	a,c
	and	0f8h
	ld	l,a
	ld	h,b
	add	hl,hl
	add	hl,de
	ld	a,l
	and	0f8h
	ld	l,a
	ld	a,c
	and	07h
	or	l
	ld	l,a
	ld	a,e
	and	07h
	inc	a
	ld	b,a
	xor	a
	scf
lf43a:	rra
	djnz	lf43a
	pop	bc
	ld	c,a
	ld	a,b
	cp	00h
	jr	nz,lf448
	ld	a,c
	or	(hl)
	ld	(hl),a
	ret		     ;f447
lf448:	cp	01h
	jr	nz,lf451
	ld	a,c
	cpl
	and	(hl)
	ld	(hl),a
	ret		     ;f450
lf451:	cp	02h
	jr	nz,lf459
	ld	a,c
	xor	(hl)
	ld	(hl),a
	ret		     ;f458
lf459:	cp	03h
	ret	nz
	ld	a,c
	and	(hl)
	ld	(MODE),a
	ret		     ;f461

; ----------------------------------------------------
; Symbol
; ----------------------------------------------------
symb:
	ld	bc,symb1     ;f462
	call	xbios
	dw	screen		;XBIOS-Funktion
	ret
symb1:	ld	hl,buffer    ;f46b
	ld	bc,0008h
	ld	de,0000h
	ldir
	ret

; ----------------------------------------------------
; Buffer for screen access
; ----------------------------------------------------
buffer: dw	0	     ;f477
	dw	0
	dw	0
	dw	0
; ----------------------------------------------------
; Get
; ----------------------------------------------------
Get:
	ld	bc,get1      ;f47f
	call	xbios
	dw	screen		;XBIOS-Funktion
	ret
get1:	ld	hl,0000h     ;f488
	ld	bc,0008h
	ld	de,buffer
	ldir	
	ret
; ----------------------------------------------------
; Screen access
; ----------------------------------------------------
ScrAcc:
	ld	bc,sa2	     ;f494
	call	xbios
	dw	screen		;XBIOS-Funktion
	ret
sa2:	ld	e,00h	     ;f49d
	ld	a,00h
	ld	d,00h
	rlca	
	rla	
	rla	
	rla	
	ld	l,a
	ld	a,d
	adc	a,0b6h
	ld	h,a
	ld	a,(hl)
	inc	hl
	ld	h,(hl)
	ld	l,a
	ex	de,hl
	add	hl,hl
	add	hl,hl
	add	hl,de
	add	hl,hl
	ld	bc,0008h
	ld	a,00h
	cp	00h
	jr	nz,lf4c4
	ld	de,0f477h
	ldir	
	ret	
lf4c4:	ex	de,hl	     ;f4c4
	ld	hl,buffer
	ldir	
	ret		     ;f4ca Ende KERNEL-MC

; ----------------------------------------------------
; Draw a line from Px(x0,y0) to Px(x1,y1)
;
; Entry:   x0     y0     x1     y1    mode
; Exit:    --
; ----------------------------------------------------
VarLine:
X0:	dw	0
Y0:	dw	0
MODE:	db	0
X1:	dw	0
Y1:	dw	0
XE:	dw	0
YE:	dw	0
DX:	dw	0
DY:	dw	0
AA:	dw	0
BB:	dw	0
FZ:	dw	0
xstep:	dw	0
ystep:	dw	0

Line:	ld	bc,start2
	call	xbios
	dw	screen		;XBIOS-Funktion
	ret
start2:
	;XE=X1
	ld	de,(X1)
	ld	(XE),de
	;DX=X1-X0
	ld	hl,(X1)
	ld	de,(X0)
	or	a
	sbc	hl,de
	ld	(DX),hl

	;YE=Y1
	ld	de,(Y1)
	ld	(YE),de

	;DY=Y1-Y0
	ld	hl,(Y1)
	ld	de,(Y0)
	or	a
	sbc	hl,de
	ld	(DY),hl

	;preset: xstep=1; ystep=1
	ld	hl,0001
	ld	(xstep),hl
	ld	(ystep),hl

;determine in witch quadrant we are working...
;IF DX < 0 THEN DX = -DX; xstep = -1
TstDX:	ld	a,(DX+1)	;get HByte of DX
	bit	7,a
	jr	z,TstDY

	;xstep = -1
	ld	hl,-1
	ld	(xstep),hl

	;DX = -DX
	ld	hl,0
	ld	de,(DX)
	or	a
	sbc	hl,de
	ld	(DX),hl

;IF DY < 0 THEN DX = -DX; ystep = -1
TstDY:	ld	a,(DY+1)	;get HByte of DY
	bit	7,a
	jr	z,LPIncX

	;ystep = -1
	ld	hl,-1
	ld	(ystep),hl

	;DY = -DY
	ld	hl,0
	ld	de,(DY)
	or	a
	sbc	hl,de
	ld	(DY),hl

;Set the linepath increment...
;AA=DX+DX
LPIncX: ld	hl,(DX)
	add	hl,hl
	ld	(AA),hl

;BB=DY+DY
LPIncY: ld	hl,(DY)
	add	hl,hl
	ld	(BB),hl

;Calculate wether we should go on in Y or X direction...
;IF DY <= DX THEN FZ = -DX; WHILE ...
	ld	hl,(DY)
	ld	de,(DX)
	inc	de		;carry only if DX > DY
	or	a
	sbc	hl,de
	jr	c,_FZMDX
	jp	ELSE1		;Goto ELSE1

	;FZ = -DX
_FZMDX: ld	de,(DX)
	ld	hl,0
	or	a
	sbc	hl,de
	ld	(FZ),hl

;WHILE X0 <> XE
_WHILE1:
	ld	hl,(X0)
	ld	de,(XE)
	or	a
	sbc	hl,de
	ld	a,h
	or	l
	jr	z,WEND1 	;IF X0 = XE THEN Goto WEND1

;Plot Point and prepare next round...
	call	start1		;Plot Pixel
	;FZ = FZ + BB
	ld	de,(FZ)
	ld	hl,(BB)
	add	hl,de
	ld	(FZ),hl

;IF FZ > 0 THEN
	bit	7,h
	jr	nz,ENDIF1	;IF FZ < 0 THEN Goto ENDIF1
	ld	a,h
	or	l
	jr	z,ENDIF1	;IF FZ = 0 THEN Goto ENDIF1

	;FZ = FZ - AA
	ld	hl,(FZ)
	ld	de,(AA)
	or	a
	sbc	hl,de
	ld	(FZ),hl

	;Y0 = Y0 + YSTEP
	ld	hl,(Y0)
	ld	de,(ystep)
	add	hl,de
	ld	(Y0),hl
ENDIF1:
	;X0 = X0 + xstep
	ld	hl,(X0) 	;X0
	ld	de,(xstep)	;xstep
	add	hl,de
	ld	(X0),hl

	jp	_WHILE1 	;Next turne
WEND1:	jp	WEND2		;Goto EndLine, we are ready...

ELSE1:
	; FZ = -DY
	ld	hl,0
	ld	de,(DY)
	or	a
	sbc	hl,de
	ld	(FZ),hl

;WHILE Y0 <> YE
_WHILE2:
	ld	hl,(Y0) 	;Y0
	ld	de,(YE) 	;YE
	or	a
	sbc	hl,de		;Y0 - YE
	ld	a,h
	or	l
	jr	z,WEND2 	;IF Y0 = YE THEN Goto WEND2

;Plot Point and prepare next round...
	call	start1		;Plot Pixel
	;FZ = FZ + AA
	ld	hl,(FZ)
	ld	de,(AA)
	add	hl,de
	ld	(FZ),hl

;IF FZ > 0 THEN
	bit	7,h
	jr	nz,ENDIF2	;IF bit[15] = 1 THEN Goto ENDIF2
	ld	a,l
	or	h
	jr	z,ENDIF2	;IF FZ = 0 THEN Goto ENDIF2

	;FZ = FZ - BB
	ld	hl,(FZ)
	ld	de,(BB)
	or	a
	sbc	hl,de
	ld	(FZ),hl 	;FZ = FZ - BB

	;X0 = X0 + xstep
	ld	hl,(X0) 	;X0
	ld	de,(xstep)	;xstep
	add	hl,de
	ld	(X0),hl 	;X0 = X0 + xstep

ENDIF2:
	;Y0 = Y0 + ystep
	ld	hl,(Y0)
	ld	de,(ystep)
	add	hl,de
	ld	(Y0),hl
	jp	_WHILE2 	;next turne
WEND2:
	call	start1		;Set last Pixel
LnRetrn:
	RET
